@@ -24,7 +24,7 @@
 /*                           Interdisciplinary Graduate School of    */
 /*                           Science and Engineering                 */
 /*                                                                   */
-/*                1996-2011  Nagoya Institute of Technology          */
+/*                1996-2013  Nagoya Institute of Technology          */
 /*                           Department of Computer Science          */
 /*                                                                   */
 /* All rights reserved.                                              */
@@ -58,7 +58,11 @@
 /* POSSIBILITY OF SUCH DAMAGE.                                       */
 /* ----------------------------------------------------------------- */
 
-static char *rcs_id = "$Id: jkGetF0.c,v 1.7 2011/12/06 09:30:00 sawada11 Exp $";
+/****************************************************************
+
+    $Id: jkGetF0.c,v 1.13 2013/12/16 09:02:02 mataki Exp $
+
+*****************************************************************/
 
 #if 0
 #include "snack.h"
@@ -73,7 +77,11 @@ static char *rcs_id = "$Id: jkGetF0.c,v 1.7 2011/12/06 09:30:00 sawada11 Exp $";
 # define FALSE 0
 #endif
 #ifndef FLT_MAX
+#if 0
 # define FLT_MAX (3.40282347E+38f) 
+#else
+# define FLT_MAX (3.40282346E+38f) 
+#endif
 #endif
 #ifndef M_PI
 # define M_PI (3.1415926536f)
@@ -377,10 +385,8 @@ check_f0_params(F0_params *par, double sample_freq)
   return(error);
 }
 
-#if 0
 static void get_cand(), peak(), do_ffir();
 static int lc_lin_fir(), downsamp();
-#endif /* 0 */
 
 /* ----------------------------------------------------------------------- */
 void get_fast_cands(fdata, fdsdata, ind, step, size, dec, start, nlags, engref, maxloc, maxval, cp, peaks, locs, ncand, par)
@@ -517,8 +523,12 @@ float *downsample(input,samsin,state_idx,freq,samsout,decimate, first_time, last
       return(foutput);
     } else
       Fprintf(stderr,"Problems in downsamp() in downsample()\n");
+#if 0  /* skip final frame if it is too small */
   } else
     Fprintf(stderr,"Bad parameters passed to downsample()\n");
+#else
+  }
+#endif
   
   return(NULL);
 }
@@ -869,7 +879,10 @@ init_dp_f0(freq, par, buffsize, sdstep)
   int nframes;
   int i;
   int stat_wsize, agap, ind, downpatch;
-
+#if 0
+#else
+  float *fgetmem(const int leng);
+#endif
 /*
  * reassigning some constants 
  */
@@ -1031,9 +1044,7 @@ init_dp_f0(freq, par, buffsize, sdstep)
   return(0);
 }
 
-#if 0
 static Stat *get_stationarity();
-#endif
 
 /*--------------------------------------------------------------------*/
 int
@@ -1067,6 +1078,10 @@ dp_f0(fdata, buff_size, sdstep, freq,
     dsdata = fdata;
   else {
     samsds = ((nframes-1) * step + ncomp) / decimate;
+#if 1 /* skip final frame if it is too small */
+    if(samsds < 1)
+      return 1; 
+#endif
     dsdata = downsample(fdata, buff_size, sdstep, freq, &samsds, decimate, 
 			first_time, last_time);
     if (!dsdata) {
@@ -1840,9 +1855,9 @@ int
 cGet_f0(Sound *sound, Tcl_Interp *interp, float **outlist, int *length)
 {
 #else
-const char *cGet_f0(float_list *input, float sample_freq, int length,
-                    int frame_shift, int minF0, int maxF0, int fnum, int otype)
+void rapt(float_list *input, int length, double sample_freq, int frame_shift, double minF0, double maxF0, double voice_bias, int otype)
 {
+  int fnum = 0;
 #endif
   float *fdata;
   int done;
@@ -1865,7 +1880,11 @@ const char *cGet_f0(float_list *input, float sample_freq, int length,
   int count = 0;
   int startpos = 0, endpos = -1;
   long max;
-  float_list *tmpf;
+  float_list *tmpf, *cur = NULL, *prev = NULL;
+  void usage(int status);
+  double p, fsp, alpha, beta;
+  unsigned long next = 1;
+  double nrandom(unsigned long *next);
 #endif /* 0 */
 
 #if 0
@@ -1876,6 +1895,30 @@ const char *cGet_f0(float_list *input, float sample_freq, int length,
 
   par = (F0_params *) ckalloc(sizeof(F0_params));
 #else
+
+  for (i = 0, tmpf = input; tmpf != NULL; i++, tmpf = tmpf->next) {
+      p = (double) nrandom(&next);
+      tmpf->f += (float) (p * 50.0);
+      prev = tmpf;
+  }
+
+  fnum = (int) (ceil((double) length / (double) frame_shift));
+  fsp = sample_freq * (10.0 / (double) frame_shift);
+  alpha = (int) (0.00275 * fsp + 0.5);
+  beta = (int) ((9600.0 / minF0 - 168.0) * fsp / 96000.0 + 0.5);
+  if (beta < 0) {
+     beta = 0;
+  }
+  for (i = 0; i < (alpha + beta + 3) * frame_shift; i++) {
+      p = (double) nrandom(&next);
+      cur = (float_list *) malloc(sizeof(float_list));
+      cur->f = (float) (p * 50.0);
+      length++;
+      prev->next = cur;
+      cur->next = NULL;
+      prev = cur;
+  }
+
   par = (F0_params *) malloc(sizeof(F0_params));
   buf = (float *) malloc(sizeof(float) * length);
   tmp = (float *) malloc(sizeof(float)
@@ -1909,6 +1952,9 @@ const char *cGet_f0(float_list *input, float sample_freq, int length,
   par->mean_f0 = 200;          /* unused */
   par->mean_f0_weight = 0.0f;  /* unused */
   par->conditioning = 0;       /* unused */
+#if 1
+  par->voice_bias = voice_bias; /* overwrite U/V threshold for pitch command */
+#endif
 
 #if 0
   if (startpos < 0) startpos = 0;
@@ -1955,18 +2001,33 @@ const char *cGet_f0(float_list *input, float sample_freq, int length,
     start_time = 0.0f;
 
     if (check_f0_params(par, sf)) {
+#endif /* 0 */
+#if 0
        return "invalid/inconsistent parameters -- exiting.";
+#else
+       fprintf(stderr, "invalid/inconsistent parameters -- exiting.\n");
+       usage(1);
+#endif
     }
 
     total_samps = endpos - startpos + 1;
 
     if (total_samps < ((par->frame_step * 2.0) + par->wind_dur) * sf) {
+#if 0
         return "input range too small for analysis by get_f0.";
+#else
+       fprintf(stderr, "input range too small for analysis by get_f0.\n");
+       usage(1);
+#endif
     }
 
     if (init_dp_f0(sf, par, &buff_size, &sdstep)
         || buff_size > INT_MAX || sdstep > INT_MAX) {
+#if 0
         return "problem in init_dp_f0().";
+#else
+       fprintf(stderr, "problem in init_dp_f0().\n");
+       usage(1);
     }
 #endif /* 0 */
 
@@ -2008,7 +2069,12 @@ const char *cGet_f0(float_list *input, float sample_freq, int length,
         }
         if (dp_f0(fdata, (int) actsize, (int) sdstep, sf, par,
                   &f0p, &vuvp, &rms_speech, &acpkp, &vecsize, done)) {
+#endif
+#if 0
             return "problem in dp_f0().";
+#else
+            break;  /* skip final frame if it is too small */
+#endif /* 0 */
         }
 
         for (i = vecsize - 1; i >= 0; i--) {
@@ -2016,7 +2082,6 @@ const char *cGet_f0(float_list *input, float sample_freq, int length,
             unvoiced[count] = vuvp[i];
             count++;
         }
-#endif /* 0 */
 
 
     if (done) break;
@@ -2084,6 +2149,5 @@ const char *cGet_f0(float_list *input, float sample_freq, int length,
 
   free_dp_f0();
 
-  return NULL;
 #endif /* 0 */
 }
