@@ -8,7 +8,7 @@
 /*                           Interdisciplinary Graduate School of    */
 /*                           Science and Engineering                 */
 /*                                                                   */
-/*                1996-2011  Nagoya Institute of Technology          */
+/*                1996-2013  Nagoya Institute of Technology          */
 /*                           Department of Computer Science          */
 /*                                                                   */
 /* All rights reserved.                                              */
@@ -44,11 +44,12 @@
 
 /****************************************************************
 
-    $Id: _smcep.c,v 1.19 2011/04/27 13:46:43 mataki Exp $
+    $Id: _smcep.c,v 1.26 2013/12/16 09:02:03 mataki Exp $
 
     Mel-Cepstral Analysis (2nd order all-pass filter)
 
-        int smcep(xw, flng, mc, m, fftsz, a, t, itr1, itr2, dd, e, itype);
+        int smcep(xw, flng, mc, m, fftsz, a, t, itr1, itr2, dd, 
+                  etype, e, f, itype);
 
         double   *xw   : input sequence
         int      flng  : frame length
@@ -59,7 +60,11 @@
         int      itr1  : minimum number of iteration
         int      itr2  : maximum number of iteration
         double   dd    : end condition
-        double   e     : initial value for log-periodgram
+        int      etype :    0 -> e is not used
+                            1 -> e is initial value for log-periodogram
+                            2 -> e is floor periodogram in db
+        double   e     : initial value for log-periodogram
+                         or floor periodogram in db 
         double   f     : mimimum value of the determinant 
                          of the normal matrix
         int      itype : input data type
@@ -552,14 +557,30 @@ static void frqtr2(double *c1, int m1, double *c2, int m2, int fftsz, double a,
 
 int smcep(double *xw, const int flng, double *mc, const int m, const int fftsz,
           const double a, const double t, const int itr1, const int itr2,
-          const double dd, const double e, const double f, const int itype)
+          const double dd, const int etype, const double e, const double f,
+          const int itype)
 {
    int i, j;
    int flag = 0, f2, m2;
-   double u, s;
+   double u, s, eps = 0.0, min, max;
    static double *x = NULL, *y, *c, *d, *al, *b;
    static int size_x, size_d;
 
+   if (etype == 1 && e < 0.0) {
+      fprintf(stderr, "smcep : value of e must be e>=0!\n");
+      exit(1);
+   }
+
+   if (etype == 2 && e >= 0.0) {
+      fprintf(stderr, "smcep : value of E must be E<0!\n");
+      exit(1);
+   }
+
+   if (etype == 1) {
+      eps = e;
+   }
+
+
    if (x == NULL) {
       x = dgetmem(3 * flng);
       y = x + flng;
@@ -595,39 +616,55 @@ int smcep(double *xw, const int flng, double *mc, const int m, const int fftsz,
    case 0:                     /* windowed data sequence */
       fftr(x, y, flng);
       for (i = 0; i < flng; i++) {
-         x[i] = x[i] * x[i] + y[i] * y[i] + e;  /*  periodegram  */
+         x[i] = x[i] * x[i] + y[i] * y[i] + eps;        /*  periodogram  */
       }
       break;
    case 1:                     /* dB */
       for (i = 0; i <= flng / 2; i++) {
-         x[i] /= 20.0 / log(10.0);      /* dB -> amplitude spectrum */
-         x[i] = x[i] * x[i] + e;        /* amplitude -> periodgram */
+         x[i] = exp((x[i] / 20.0) * log(10.0)); /* dB -> amplitude spectrum */
+         x[i] = x[i] * x[i] + eps;      /* amplitude -> periodogram */
       }
       break;
    case 2:                     /* log */
       for (i = 0; i <= flng / 2; i++) {
          x[i] = exp(x[i]);      /* log -> amplitude spectrum */
-         x[i] = x[i] * x[i] + e;        /* amplitude -> periodgram */
+         x[i] = x[i] * x[i] + eps;      /* amplitude -> periodogram */
       }
       break;
    case 3:                     /* amplitude */
       for (i = 0; i <= flng / 2; i++) {
-         x[i] = x[i] * x[i] + e;        /* amplitude -> periodgram */
+         x[i] = x[i] * x[i] + eps;      /* amplitude -> periodogram */
       }
       break;
-   case 4:                     /* periodgram */
+   case 4:                     /* periodogram */
       for (i = 0; i <= flng / 2; i++) {
-         x[i] = x[i] + e;
+         x[i] = x[i] + eps;
       }
       break;
    default:
-      fprintf(stderr, "mgcep : Input type %d is not supported!\n", itype);
+      fprintf(stderr, "smcep : Input type %d is not supported!\n", itype);
       exit(1);
    }
    if (itype > 0) {
       for (i = 1; i < flng / 2; i++)
          x[flng - i] = x[i];
    }
+
+   if (etype == 2 && e < 0.0) {
+      max = x[0];
+      for (i = 1; i < flng; i++) {
+         if (max < x[i])
+            max = x[i];
+      }
+      max = sqrt(max);
+      min = max * pow(10.0, e / 20.0);  /* floor is 20*log10(min/max) */
+      min = min * min;
+      for (i = 0; i < flng; i++) {
+         if (x[i] < min)
+            x[i] = min;
+      }
+   }
+
    for (i = 0; i < flng; i++)
       c[i] = log(x[i]);
 
