@@ -8,7 +8,7 @@
 /*                           Interdisciplinary Graduate School of    */
 /*                           Science and Engineering                 */
 /*                                                                   */
-/*                1996-2011  Nagoya Institute of Technology          */
+/*                1996-2013  Nagoya Institute of Technology          */
 /*                           Department of Computer Science          */
 /*                                                                   */
 /* All rights reserved.                                              */
@@ -48,6 +48,7 @@
 *                                                                       *
 *                                      1998.12 T.Masuko                 *
 *                                      2010.8  A.Tamamori               *
+*                                      2012.11 T.Okada                  *
 *       usage:                                                          *
 *               vstat [ options ] [ infile ] > stdout                   *
 *       options:                                                        *
@@ -60,9 +61,10 @@
 *                             2 covariance                              *
 *                             3 mean and upper / lower bound of         *
 *                               confidence interval via t-dist.         *
+*                             4 median                                  *
 *               -c c     :  conf. level of conf. interval      [0.95]   *
 *               -d       :  diagonal covariance                [FALSE]  *
-*               -i       :  output inverse cov. instead of cov.[FALSE]  * 
+*               -i       :  output inverse cov. instead of cov.[FALSE]  *
 *               -r       :  output correlation instead of cov. [FALSE]  *
 *       infile:                                                         *
 *              data sequence                                            *
@@ -86,7 +88,7 @@
 *                                                                       *
 ************************************************************************/
 
-static char *rcs_id = "$Id: vstat.c,v 1.26 2011/04/27 13:46:44 mataki Exp $";
+static char *rcs_id = "$Id: vstat.c,v 1.35 2013/12/16 09:02:06 mataki Exp $";
 
 
 /*  Standard C Libralies  */
@@ -116,6 +118,7 @@ static char *rcs_id = "$Id: vstat.c,v 1.26 2011/04/27 13:46:44 mataki Exp $";
 #define OUTMEAN TR
 #define OUTCOV  TR
 #define OUTCONF FA
+#define OUTMED  FA
 #define CONFLEV 0.9500E2
 #define DIAGC   FA
 #define INV     FA
@@ -126,7 +129,6 @@ char *BOOL[] = { "FALSE", "TRUE" };
 /*  Command Name  */
 char *cmnd;
 
-
 void usage(int status)
 {
    fprintf(stderr, "\n");
@@ -147,7 +149,9 @@ void usage(int status)
    fprintf(stderr, "                  2 covariance\n");
    fprintf(stderr, "                  3 mean and upper / lower bound of\n");
    fprintf(stderr, "                    confidence interval via t-dist.\n");
-   fprintf(stderr, "       -c c   : conf. level of conf. interval (%)   [%g]\n",
+   fprintf(stderr, "                  4 median\n");
+   fprintf(stderr,
+           "       -c c   : conf. level of conf. interval (%%)   [%g]\n",
            CONFLEV);
    fprintf(stderr, "       -d     : diagonal covariance                 [%s]\n",
            BOOL[DIAGC]);
@@ -261,14 +265,49 @@ double t_percent(const double p, const int df)
    return (x);
 }
 
+void quicksort(double *x, int left, int right)
+{
+   int i, j;
+   int pivot;
+   int tmp;
+
+   i = left;
+   j = right;
+
+   pivot = x[(left + right) / 2];
+
+   while (1) {
+
+      while (x[i] < pivot)
+         i++;
+
+      while (pivot < x[j])
+         j--;
+      if (i >= j)
+         break;
+
+      tmp = x[i];
+      x[i] = x[j];
+      x[j] = tmp;
+
+      i++;
+      j--;
+   }
+
+   if (left < i - 1)
+      quicksort(x, left, i - 1);
+   if (j + 1 < right)
+      quicksort(x, j + 1, right);
+}
+
 int main(int argc, char *argv[])
 {
    FILE *fp = stdin;
-   double *x, *mean, **cov = NULL, **invcov = NULL, *var = NULL,
-       conf = CONFLEV, *upper, *lower, t, err;
-   int leng = LENG, nv = -1, i, j, k, lp, m, outtype = 0;
+   double *x, *mean, *med = NULL, **mtmp = NULL, **cov = NULL, **invcov =
+       NULL, *var = NULL, conf = CONFLEV, *upper = NULL, *lower = NULL, t, err;
+   int leng = LENG, nv = -1, i, j, k = 0, lp = 0, m, outtype = 0, count = 0;
    Boolean outmean = OUTMEAN, outcov = OUTCOV, outconf = OUTCONF,
-       diagc = DIAGC, inv = INV, corr = CORR;
+       outmed = OUTMED, diagc = DIAGC, inv = INV, corr = CORR;
 
    if ((cmnd = strrchr(argv[0], '/')) == NULL)
       cmnd = argv[0];
@@ -331,6 +370,11 @@ int main(int argc, char *argv[])
       outcov = FA;
       outconf = TR;
       break;
+   case 4:
+      outcov = FA;
+      outmean = FA;
+      outmed = TR;
+      break;
    }
    if (diagc && corr)
       diagc = FA;
@@ -340,7 +384,83 @@ int main(int argc, char *argv[])
       corr = FA;
 
    mean = dgetmem(leng + leng);
+
    x = mean + leng;
+
+   if (outmed) {
+
+      if (nv == -1) {
+         typedef struct _float_list {
+            float *f;
+            struct _float_list *next;
+         } float_list;
+
+         float_list *top = NULL, *prev = NULL, *cur = NULL;
+         top = prev = (float_list *) getmem(1, sizeof(float_list));
+         prev->next = NULL;
+         while (freadf(x, sizeof(*x), leng, fp) == leng) {
+            cur = (float_list *) getmem(1, sizeof(float_list));
+            cur->f = fgetmem(leng);
+            for (i = 0; i < leng; i++) {
+               cur->f[i] = (float) x[i];
+            }
+            count++;
+            prev->next = cur;
+            cur->next = NULL;
+            prev = cur;
+         }
+         k = count;
+
+         mtmp = (double **) getmem(leng, sizeof(*mtmp));
+         mtmp[0] = dgetmem(leng * k);
+         for (i = 1; i < leng; i++)
+            mtmp[i] = mtmp[i - 1] + k;
+
+         med = dgetmem(leng);
+
+         for (j = 0, cur = top->next; j < k; j++, cur = cur->next) {
+            for (i = 0; i < leng; i++) {
+               mtmp[i][j] = (double) cur->f[i];
+            }
+         }
+      } else {
+
+         k = nv;
+
+         mtmp = (double **) getmem(leng, sizeof(*mtmp));
+         mtmp[0] = dgetmem(leng * k);
+         for (i = 1; i < leng; i++)
+            mtmp[i] = mtmp[i - 1] + k;
+
+         med = dgetmem(leng);
+
+         for (j = 0; j < k; j++) {
+            for (i = 0; i < leng; i++) {
+               freadf(&mtmp[i][j], sizeof(**mtmp), 1, fp);
+            }
+         }
+      }
+
+      if (k % 2 == 0) {
+         fprintf(stderr, "%s : warning: the number of vectors is even!\n",
+                 cmnd);
+      }
+
+      for (i = 0; i < leng; i++) {
+         quicksort(mtmp[i], 0, k - 1);
+         if (k % 2 == 1) {
+            med[i] = mtmp[i][k / 2];
+         } else {
+            med[i] = ((mtmp[i][k / 2] + mtmp[i][k / 2 - 1]) / 2);
+         }
+      }
+
+      fwritef(med, sizeof(*med), leng, stdout);
+
+      return (0);
+
+   }
+
    if (outcov) {
       if (!diagc) {
          cov = (double **) getmem(leng, sizeof(*cov));
@@ -363,6 +483,8 @@ int main(int argc, char *argv[])
       lower = dgetmem(leng);
    }
 
+
+
    while (!feof(fp)) {
       for (i = 0; i < leng; i++) {
          mean[i] = 0.0;
@@ -377,6 +499,7 @@ int main(int argc, char *argv[])
             var[i] = 0.0;
          }
       }
+
       for (lp = nv; lp;) {
          if (freadf(x, sizeof(*x), leng, fp) != leng)
             break;
@@ -395,6 +518,7 @@ int main(int argc, char *argv[])
          }
          --lp;
       }
+
       if (lp == 0 || nv == -1) {
          if (nv > 0)
             k = nv;
@@ -422,6 +546,7 @@ int main(int argc, char *argv[])
                lower[i] = mean[i] - err;
             }
          }
+
          if (corr) {
             for (i = 0; i < leng; i++)
                for (j = i + 1; j < leng; j++)
@@ -473,6 +598,7 @@ int main(int argc, char *argv[])
             fwritef(lower, sizeof(*lower), leng, stdout);
          }
       }
+
    }
 
    return (0);
